#include "U8glib.h"
#include <SoftwareSerial.h>

// canale de comunicatie pentru modulul bluetooth
SoftwareSerial BTserial(10, 11); // RX | TX

// pini pentru ecranul OLED
U8GLIB_SH1106_128X64 u8g(5, 9, 12, 4, 13); // D0=5, D1=9, CS=12, DC=4, Reset=13

// harta de biti ce reprezinta desenul de pe OLED
const uint8_t earth_bitmap[] PROGMEM = {
 0x00, 0x00, 0x03, 0xB0, 0x00, 0x00, 0x00, 0x00, 0x07, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x46,
0x00, 0x00, 0x00, 0x00, 0xFC, 0x47, 0xC0, 0x00, 0x00, 0x01, 0xCE, 0x4C, 0x60, 0x00, 0x00, 0x03,
0x02, 0x58, 0x30, 0x00, 0x00, 0x03, 0x02, 0x58, 0x10, 0x00, 0x00, 0x02, 0x02, 0x58, 0x18, 0x00,
0x00, 0x03, 0x06, 0x4C, 0x18, 0x00, 0x00, 0x07, 0x04, 0x44, 0x18, 0x00, 0x00, 0x0D, 0x80, 0x40,
0x3C, 0x00, 0x00, 0x09, 0xC0, 0x40, 0xE6, 0x00, 0x00, 0x18, 0x78, 0x47, 0xC2, 0x00, 0x00, 0x18,
0x0C, 0x4E, 0x02, 0x00, 0x00, 0x1F, 0x86, 0x4C, 0x7E, 0x00, 0x00, 0x0E, 0xC6, 0xE8, 0xEE, 0x00,
0x00, 0x18, 0x43, 0xF8, 0x82, 0x00, 0x00, 0x10, 0x06, 0x4C, 0x03, 0x00, 0x00, 0x30, 0x0C, 0x46,
0x01, 0x00, 0x00, 0x30, 0x18, 0x46, 0x01, 0x00, 0x00, 0x10, 0x18, 0x43, 0x03, 0x00, 0x00, 0x18,
0x10, 0x43, 0x03, 0x00, 0x00, 0x1C, 0x70, 0x41, 0x86, 0x00, 0x00, 0x0F, 0xE0, 0x40, 0xFE, 0x00,
0x00, 0x09, 0x1E, 0x4F, 0x06, 0x00, 0x00, 0x08, 0x30, 0x43, 0x86, 0x00, 0x00, 0x0C, 0x20, 0x41,
0x86, 0x00, 0x00, 0x06, 0x60, 0x40, 0x8C, 0x00, 0x00, 0x07, 0x60, 0x40, 0xB8, 0x00, 0x00, 0x01,
0xE0, 0x41, 0xF0, 0x00, 0x00, 0x00, 0x38, 0xE3, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xBE, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F, 0xCF, 0x82, 0x0C, 0x86, 0x46, 0x1F, 0xEF, 0xC3, 0x0C,
0xC6, 0xEE, 0x1C, 0xEC, 0xC7, 0x0C, 0xE6, 0x7C, 0x1C, 0xED, 0x8D, 0x8C, 0xFE, 0x38, 0x1C, 0xED,
0x8D, 0xCC, 0xDE, 0x38, 0x1D, 0xCD, 0xDF, 0xCC, 0xCE, 0x38, 0x1F, 0x8C, 0xF8, 0xEC, 0xC6, 0x38,
0x1F, 0xEC, 0x08, 0x0C, 0xC2, 0x18, 0x1C, 0xEC, 0x00, 0xC0, 0x00, 0x00, 0x1C, 0xFD, 0xFB, 0xC0,
0x00, 0x00, 0x1C, 0xFC, 0x63, 0x00, 0x00, 0x00, 0x1C, 0xEC, 0x63, 0xC0, 0x00, 0x00, 0x1F, 0xEC,
0x60, 0xC0, 0x00, 0x00, 0x1F, 0xCC, 0x63, 0xC0, 0x00, 0x00, 0x1F, 0x0C, 0x63, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x2B, 0x4F, 0x67,
0x42, 0x38, 0x7B, 0xEA, 0x86, 0xB2, 0x28, 0xC7,
};

// pini RGB
int red = 8;
int blue = 6;
int green = 7;

// pin Buzzer
int buzzer = 3;

// functie ce afiseaza datele masurate de senzor pe OLED
// si care trimite aceste date pe telefon prin Bluetooth
void draw(int airQualityValue, int lightIntensityValue) {
  // afisam o imagine in partea dreapta a ecranului
  u8g.drawBitmapP( 79, 5, 6, 50, earth_bitmap);
  // afisam datele masurate pe restul ecranului
  u8g.setFont(u8g_font_unifont);  // selecteaza fontul
  u8g.drawStr(0, 10, "Con: ");  // pune string-ul la pozitia X, Y
  u8g.setPrintPos(34, 10);  // seteaza pozitia
  u8g.print(airQualityValue);
  u8g.drawStr(62, 10, "PPM");
  u8g.drawStr(0, 30, "Int: ");
  u8g.setPrintPos(34, 30);
  u8g.print(lightIntensityValue);
  u8g.drawStr(62, 30, "can");

  // in functie de acest scor se aprinde LED-ul in verde,
  // albastru, respectiv rosu, iar in ultimele doua cazuri
  // porneste si buzzer-ul la diferite frecvente
  int compare = airQualityValue + lightIntensityValue;
  if (compare >= 230) {
    digitalWrite(green, HIGH);
    digitalWrite(blue, LOW);
    digitalWrite(red, LOW);
    noTone(buzzer);
  } else if (compare >= 210 && compare < 230) {
    digitalWrite(green, LOW);
    digitalWrite(blue, HIGH);
    digitalWrite(red, LOW);
    tone(buzzer, 1000);
  } else if (compare < 210) {
    digitalWrite(green, LOW);
    digitalWrite(blue, LOW);
    digitalWrite(red, HIGH);
    tone(buzzer, 2000);
  }

  // trimite datele pe telefon
  BTserial.print(airQualityValue);
  BTserial.print(",");
  BTserial.print(lightIntensityValue);
  BTserial.print(";");
}

void setup(void) {
  // setam pinii de iesire
  pinMode(red, OUTPUT);
  pinMode(blue, OUTPUT);
  pinMode(green, OUTPUT);
  pinMode(buzzer, OUTPUT);
  BTserial.begin(9600);
  Serial.begin(9600); // sets the serial port to 9600
}

void loop(void) {
  // citim datele de la senzori
  int airQualityValue = analogRead(0); // read analog input pin 0
  int lightIntensityValue = analogRead(1);

  // afisam incontinuu pe ecran
  u8g.firstPage();  
  do {
    draw(airQualityValue, lightIntensityValue);
  } while( u8g.nextPage() );
  
  delay(1000);  // Delay of 5sec before accessing DHT11 (min - 2sec)
}
